==> ircii-2.8.51-pana.tar.gz: ircii-2.8/source/edit2.c <==


#include "irc.h"

#include <sys/stat.h>

#ifdef ESIX
# include <lan/net_types.h>
#endif

#include "parse.h"
#include "ircterm.h"
#include "server.h"
#include "chelp.h"
#include "edit.h"
#include "crypt.h"
#include "vars.h"
#include "ircaux.h"
#include "lastlog.h"
#include "window.h"
#include "screen.h"
#include "whois.h"
#include "hook.h"
#include "input.h"
#include "ignore.h"
#include "keys.h"
#include "names.h"
#include "alias.h"
#include "history.h"
#include "funny.h"
#include "ctcp.h"
#include "dcc.h"
#include "translat.h"
#include "output.h"
#include "exec.h"
#include "notify.h"
#include "numbers.h"
#include "status.h"
#include "if.h"
#include "help.h"
#include "stack.h"
#include "queue.h"
#include "timer.h"
#include "list.h"
#include "pluto.h"
#include "userlist.h"
#include "whowas.h"

void nwhois (char *command, char *args, char *subargs)
{
ChannelList *tmp;
NickList *user = NULL;
int once = 0;
char *nick;
int mem_use = 0, max_mem = 0;
	if (args && *args)
	{
		tmp = server_list[from_server].chan_list;
		if (!tmp)
		{
			bitchsay("No Channel");
			return;
		}
		nick = next_arg(args, &args);
		while (tmp)
		{
			for (user = tmp->nicks; user; user = user->next)
			{
				if (wild_match(user->nick, nick) || wild_match(nick, user->nick))
				{
					if (!once)
					{
put_it("Channel    @ v  UL: l   a  p  SL: l      +o   -o   +b   -b  kicks  nicks   pub");
						once++;
					}
put_it("%-10s %c %c    %3d  %2d %2d     %2d    %4d %4d %4d %4d  %5d  %5d %5d",
tmp->channel, user->chanop? '@':' ',user->voice?'v':' ',user->userlist?user->userlist->level:0, 
user->userlist?user->userlist->aop:0,user->userlist?user->userlist->prot:0, user->shitlist?user->shitlist->level: 0,
user->stat_ops,user->stat_dops, user->stat_bans, user->stat_unbans, 
user->stat_kicks, user->stat_nicks, user->stat_pub);
					mem_use = sizeof(NickList) + (user->userlist? sizeof(UserList): 0) + (user->shitlist? sizeof(ShitList):0) + (user->botlist?sizeof(UserList):0);
					if (max_mem < mem_use)
						max_mem = mem_use;
					break;
				}
			}				
			tmp = tmp->next;
		}
		if (once)
			bitchsay("End /NWhoIs list for %s [Mem Usage: %d bytes per chan]", nick, max_mem);
		else
			bitchsay("No such nick %s on any channel", nick);
	}
	else
		bitchsay("Nwhois usage: /Nwhois nick"); 
}

void whowas _((char *command, char *args, char *subargs))
{
	if (!command)
		show_whowas();
	else
		show_wholeft();
}

void do_sk _((char *command, char *args, char *subargs))
{
char *serv = NULL;
char *to = NULL;
int count;
NickList *Nick;
ChannelList *chan;

	if (!args || !*args)
	{
		bitchsay("Usage: /SK [server] <channel>");
		return;
	}
	serv = next_arg(args, &args);
	if (*serv == '#')
	{
		bitchsay("You need to specify a server not a channel");
	}
	if (!(to = next_arg(args, &args)))
		to = get_channel_by_refnum(0);
	if (!to || !is_channel(to))
	{
		not_on_a_channel();
		return;
	}
	if (!are_you_opped(to))
	{
		you_not_opped(to);
		return;
	}
	if ((chan = lookup_channel(to, from_server, CHAN_NOUNLINK)))
	{
		bitchsay("Performing Lame Server kicks on %s", chan->channel);
		for (Nick = chan->nicks, count = 0; Nick; Nick = Nick->next)
		{
			if (Nick->server && wild_match(serv, Nick->server))
			{
				if (!isme(Nick->nick))
				{
					send_to_server("KICK %s %s : Lame Server Kick", to, Nick->nick);
					count++;
				}
			}
		}
		if (count)
			bitchsay("Found %d lamerz on %s from %s", count, to, serv);
	} else
		bitchsay("Error in /sk");
}

int check_args(char *remote_host, char *remote_ip)
{
struct hostent *host;

	host = gethostbyname(remote_host);

	if (host == NULL) 
	{
		unsigned char a,b,c,d,n;
		unsigned char addr[5];
		/* hmm.. perhaps it was a dotted quad entered.. */
		n = sscanf(remote_host,"%u.%u.%u.%u",&a,&b,&c,&d);
		if (n != 4) 
		{
			bitchsay("error: host '%s' not found",remote_host);
			return 0;
		}
		addr[0] = a;
		addr[1] = b;
		addr[2] = c;
		addr[3] = d;
		host = gethostbyaddr(addr,4,AF_INET);
		if (host == NULL) {
			bitchsay("error: host '%s' not found",remote_host);
			return 0;
		}
		sprintf(remote_ip,"%u.%u.%u.%u",a,b,c,d);
	} 
	else 
	{
		sprintf(remote_ip,"%u.%u.%u.%u", (unsigned char) host->h_addr_list[0][0],
			(unsigned char) host->h_addr_list[0][1],
			(unsigned char) host->h_addr_list[0][2],
			(unsigned char) host->h_addr_list[0][3]);
	}
	return 1;
}

int scan(char *remote_host, char *remote_ip, int low_port, int high_port)
{
	int soc;
	struct sockaddr_in addr;
	struct servent *serv;
	int port,rc,addr_len,opt;
	int found = 0;

	for (port = low_port;port <= high_port;port++) 
	{

		soc = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);

		if (soc < 0) 
		{
			bitchsay("error: socket() failed");
			return 0;
		}

		rc = setsockopt(soc,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt));

		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = inet_addr(remote_ip);
		addr.sin_port = htons(port);

		addr_len = sizeof(addr);
		rc = connect(soc, (struct sockaddr*) &addr, addr_len);

		close(soc);


		if (rc < 0)
			continue;

		serv = getservbyport(htons(port),"tcp");
/*
		else if (scan_type == s_udp)
			serv = getservbyport(htons(port),"udp");
		else
			return;
*/
		found++;
		put_it("port %d (%s) is running",port,(serv == NULL)?"UNKNOWN": serv->s_name);
	}
	if (!found)
		bitchsay("Wonder what hostname that was. No ports found");
	return 1;
}

void findports(char *command, char *args, char *subargs)
{
char *remote_host;
char remote_ip[20];
int low_port = 6660;
int high_port = 7000;
	if (args && *args)
	{
		char *tmp = NULL;
		remote_host = next_arg(args, &args);
		if (args && *args)
		{
			tmp = next_arg(args, &args);
			low_port = strtoul(tmp, NULL, 10);	
			if (args && *args)
			{
				tmp = next_arg(args, &args);
				high_port = strtoul(tmp, NULL, 10);	
			}
			else
				high_port = low_port;
		}
		if (check_args(remote_host, remote_ip))
		{
			bitchsay("Scanning %s's tcp ports %d through %d",remote_host, low_port,high_port);
			scan(remote_host, remote_ip, low_port, high_port);
		}
	} else
		bitchsay("Usage: /findports <hostname> [low port [high port]]");
	return;
}

==> ircii-2.8.56-pana.tar.gz: ircii-2.8/source/edit2.c <==
/*
 * Routines within this files are Copyright Colten Edwards 1996
 * Aka panasync on irc.
 * Thanks to Shiek and Flier for helpful hints and suggestions. As well 
 * as code in some cases.
 */
 
#include "irc.h"

#include <sys/stat.h>

#ifdef ESIX
# include <lan/net_types.h>
#endif

#include "parse.h"
#include "ircterm.h"
#include "server.h"
#include "chelp.h"
#include "edit.h"
#include "crypt.h"
#include "vars.h"
#include "ircaux.h"
#include "lastlog.h"
#include "window.h"
#include "screen.h"
#include "whois.h"
#include "hook.h"
#include "input.h"
#include "ignore.h"
#include "keys.h"
#include "names.h"
#include "alias.h"
#include "history.h"
#include "funny.h"
#include "ctcp.h"
#include "dcc.h"
#include "translat.h"
#include "output.h"
#include "exec.h"
#include "notify.h"
#include "numbers.h"
#include "status.h"
#include "if.h"
#include "help.h"
#include "stack.h"
#include "queue.h"
#include "timer.h"
#include "list.h"
#include "pluto.h"
#include "misc.h"
#include "userlist.h"
#include "whowas.h"

void nwhois (char *command, char *args, char *subargs)
{
ChannelList *tmp;
NickList *user = NULL;
int once = 0;
char *nick;
int mem_use = 0, max_mem = 0;
	if (args && *args)
	{
		tmp = server_list[from_server].chan_list;
		if (!tmp)
		{
			bitchsay("No Channel");
			return;
		}
		nick = next_arg(args, &args);
		while (tmp)
		{
			for (user = tmp->nicks; user; user = user->next)
			{
				if (match(user->nick, nick) || match(nick, user->nick))
				{
					if (!once)
					{
put_it("Channel    @ v  UL: l   a  p  SL: l      +o   -o   +b   -b  kicks  nicks   pub");
						once++;
					}
put_it("%-10s %c %c    %3d  %2d %2d     %2d    %4d %4d %4d %4d  %5d  %5d %5d",
tmp->channel, user->chanop? '@':' ',user->voice?'v':' ',user->userlist?user->userlist->level:0, 
user->userlist?user->userlist->aop:0,user->userlist?user->userlist->prot:0, user->shitlist?user->shitlist->level: 0,
user->stat_ops,user->stat_dops, user->stat_bans, user->stat_unbans, 
user->stat_kicks, user->stat_nicks, user->stat_pub);
					mem_use = sizeof(NickList) + (user->userlist? sizeof(UserList): 0) + (user->shitlist? sizeof(ShitList):0) + (user->botlist?sizeof(UserList):0);
					if (max_mem < mem_use)
						max_mem = mem_use;
					break;
				}
			}				
			tmp = tmp->next;
		}
		if (once)
			bitchsay("End /NWhoIs list for %s [Mem Usage: %d bytes per chan]", nick, max_mem);
		else
			bitchsay("No such nick %s on any channel", nick);
	}
	else
		userage("nwhois", "<nick>");
}

void whowas _((char *command, char *args, char *subargs))
{
	if (!command)
		show_whowas();
	else
		show_wholeft();
}

void do_sk _((char *command, char *args, char *subargs))
{
char *serv = NULL;
char *to = NULL;
int count;
NickList *Nick;
ChannelList *chan;

	if (!args || !*args)
	{
		userage("sk", "<server> [channel]");
		return;
	}
	serv = next_arg(args, &args);
	if (*serv == '#')
	{
		bitchsay("You need to specify a server not a channel");
	}
	if (!(to = next_arg(args, &args)))
		to = get_channel_by_refnum(0);
	if (!to || !is_channel(to))
	{
		not_on_a_channel();
		return;
	}
	if (!are_you_opped(to))
	{
		you_not_opped(to);
		return;
	}
	if ((chan = lookup_channel(to, from_server, CHAN_NOUNLINK)))
	{
		bitchsay("Performing Lame Server kicks on %s", chan->channel);
		for (Nick = chan->nicks, count = 0; Nick; Nick = Nick->next)
		{
			if (Nick->server && match(serv, Nick->server))
			{
				if (!isme(Nick->nick))
				{
					send_to_server("KICK %s %s : Lame Server Kick", to, Nick->nick);
					count++;
				}
			}
		}
		if (count)
			bitchsay("Found %d lamerz on %s from %s", count, to, serv);
	} else
		bitchsay("Error in /sk");
}

int check_args(char *remote_host, char *remote_ip)
{
struct hostent *host;

	host = gethostbyname(remote_host);

	if (host == NULL) 
	{
		unsigned char a,b,c,d,n;
		unsigned char addr[5];
		/* hmm.. perhaps it was a dotted quad entered.. */
		n = sscanf(remote_host,"%u.%u.%u.%u",&a,&b,&c,&d);
		if (n != 4) 
		{
			bitchsay("error: host '%s' not found",remote_host);
			return 0;
		}
		addr[0] = a;
		addr[1] = b;
		addr[2] = c;
		addr[3] = d;
		host = gethostbyaddr(addr,4,AF_INET);
		if (host == NULL) {
			bitchsay("error: host '%s' not found",remote_host);
			return 0;
		}
		sprintf(remote_ip,"%u.%u.%u.%u",a,b,c,d);
	} 
	else 
	{
		sprintf(remote_ip,"%u.%u.%u.%u", (unsigned char) host->h_addr_list[0][0],
			(unsigned char) host->h_addr_list[0][1],
			(unsigned char) host->h_addr_list[0][2],
			(unsigned char) host->h_addr_list[0][3]);
	}
	return 1;
}

int scan(char *remote_host, char *remote_ip, int low_port, int high_port)
{
	int soc;
	struct sockaddr_in addr;
	struct servent *serv;
	int port,rc,addr_len,opt;
	int found = 0;

	for (port = low_port;port <= high_port;port++) 
	{
		soc = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);

		if (soc < 0) 
		{
			bitchsay("error: socket() failed");
			return 0;
		}

		rc = setsockopt(soc,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt));

		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = inet_addr(remote_ip);
		addr.sin_port = htons(port);

		addr_len = sizeof(addr);
		alarm(1);
		rc = connect(soc, (struct sockaddr*) &addr, addr_len);
		alarm(0);
		close(soc);


		if (rc < 0)
			continue;

		serv = getservbyport(htons(port),"tcp");
		found++;
		put_it("port %d (%s) is running",port,(serv == NULL)?"UNKNOWN": serv->s_name);
		io();
	}
	if (!found)
		bitchsay("Wonder what hostname that was. No ports found");
	return 1;
}

void findports(char *command, char *args, char *subargs)
{
char *remote_host;
char remote_ip[20];
int low_port = 6660;
int high_port = 7000;
	if (args && *args)
	{
		char *tmp = NULL;
		remote_host = next_arg(args, &args);
		if (args && *args)
		{
			tmp = next_arg(args, &args);
			low_port = strtoul(tmp, NULL, 10);	
			if (args && *args)
			{
				tmp = next_arg(args, &args);
				high_port = strtoul(tmp, NULL, 10);	
			}
			else
				high_port = low_port;
		}
		if (check_args(remote_host, remote_ip))
		{
			bitchsay("Scanning %s's tcp ports %d through %d",remote_host, low_port,high_port);
			scan(remote_host, remote_ip, low_port, high_port);
		}
	} else
		userage("findports", "<hostname> [lowport] [high port]");
	return;
}

void add_ban_word(char *command, char *args, char *subargs)
{
char *word = NULL;
char *chan = NULL;
WordKickList *new;

	if (args && *args)
	{
		chan = next_arg(args, &args);
		if (!is_channel(chan) || !args || !*args)
		{
			userage(command ? command : "banwords", "<channel> <word/string>");
			return;
		}
		if (!command)
		{
			new = (WordKickList *)find_in_list((List **)&ban_words, args, 0);
			if (!ban_words || !new || (new && !match(new->channel, chan)))
			{
				new = (WordKickList *) new_malloc(sizeof(WordKickList));
				malloc_strcpy(&new->channel, chan);
				malloc_strcpy(&new->string, args);
				add_to_list((List **)&ban_words, (List *)new);
				bitchsay("Added %s to %s Banned Word List", new->string, new->channel);
			} else bitchsay("[%s] is already in the list for channel %s", args, chan);
		}
		else
		{
			malloc_strcpy(&word, args);
			if ((new = (WordKickList *) remove_from_list((List **)&ban_words, word)))
			{
				bitchsay("Removed %s Banned Word [%s]", new->channel, new->string);
				new_free(&new->channel);
				new_free(&new->string);
				new_free((char **)&new);
			} else 
				bitchsay("Banned Word %s not found.", word);
			new_free(&word);
		}
	} else
		userage(command ? command : "banwords", "<channel> <word/string>");
}

void show_word_kick (char *command, char *args, char *subargs)
{
WordKickList *new;
	if (ban_words) 
	{
		put_it("%11s %40s", "Channel", "Banned Word(s)");
		for (new = ban_words; new; new = new->next)
			put_it("%-11s %40s", new->channel, new->string);
	} else
		bitchsay("No Banned Words list");
}

void save_banwords(FILE *outfile)
{
	int count = 0;
	WordKickList *new;
	if (ban_words)
	{
		fprintf(outfile, "# %s Banned Words\n", version);
		for (new = ban_words; new; new = new->next)
		{
			fprintf(outfile, "BANWORD %s %s\n", new->channel, new->string);
			count++;
		}
	}
	if (do_hook(SAVEFILE_LIST,"BanWords %d", count))
		bitchsay("Saved %d Banned Words List", count);
}

void whereis (char *command, char *args, char *subargs)
{
NickList *tmp = NULL;
WhowasList *tmp1;
ChannelList *chan;
char *nick;
	if (args && *args)
	{
		nick = next_arg(args, &args);
		for (chan = server_list[from_server].chan_list; chan; chan = chan->next)
			if ((tmp = (NickList *)find_in_list((List **)&chan->nicks, nick, 0)))
				break;
		if (!tmp)
		{
			tmp1 = find_in_whowas(nick);
			if (tmp1) tmp = tmp1->nicklist;
		}
		if (tmp && tmp->server)
		{
			char *p = NULL, *user = NULL;
			malloc_strcpy(&user, tmp->host);
			p = strchr(user, '@');
			*p++ = '\0';
			send_to_server("PRIVMSG %s@%s :%cVERSION%c",user, tmp->server,CTCP_DELIM_CHAR,CTCP_DELIM_CHAR);
			new_free(&user);
		}
		else bitchsay("No such user [%s].", nick);
	} else userage("whereis", "<nick>");
}
